
using System;
using System.Text;
using System.IO;
using System.Net.Sockets;

public class Client
{
  private static int counter = 0;
  
  public static void Main ( String[] args )
  {
    if ( args.Length != 2 )
    {
      Console.WriteLine ( "Usage: Client.exe <count> <performance>" );
      Console.WriteLine ();
      Console.WriteLine ( "  <count>       - integer, the algorithm requests <count> customers, " );
      Console.WriteLine ( "                           <count>*2 contacts, and <count>*4 phone numbers" );
      Console.WriteLine ( "  <performance> - string, if value is \"slow\" the sample uses the slow " );
      Console.WriteLine ( "                          algorithm; if value is \"faster\" the sample uses " );
      Console.WriteLine ( "                          the faster algorithm" );
    }
    else
    {
      int numberOfCustomers = Convert.ToInt32 ( args[0] );
      bool slow = args[1].ToUpper ().Equals ( "SLOW" );
    
      Console.WriteLine ( "starting client, number of customers=" + numberOfCustomers + ", mode=" + (slow ? "slow" : "faster" ) + "..." );
    
      Console.WriteLine ( "connecting to 127.0.0.1:4711..." );
      TcpClient client = new TcpClient ( "127.0.0.1", 4711 );
      Console.WriteLine ( "connected to 127.0.0.1:4711" );
    
      NetworkStream output = client.GetStream ();
      StreamReader input = new StreamReader ( output );
    
      DateTime start = DateTime.Now;
    
      if ( slow )
        slowExecution ( input, output, numberOfCustomers );
      else
        fasterExecution ( input, output, numberOfCustomers );
    
      DateTime stop = DateTime.Now;
    
      Console.WriteLine ( counter + " requests, duration: " + stop.Subtract ( start ).TotalMilliseconds );
    
      client.Close ();
    }
  }
  
  public static void slowExecution ( StreamReader input, NetworkStream output, int numberOfCustomers )
  {
    String[] resp = null;
    for ( int i = 0; i < numberOfCustomers; i++ )
    {
      // get customer i
      Console.WriteLine ( sendRequest ( input, output, "CUSTOMER:" + i )[0] );
      
      for ( int j = 0; j < numberOfCustomers * 2; j++ )
      {
        // get contact j
        resp = sendRequest ( input, output, "CONTACT:" + j );
        int contactCustomerIndex = Convert.ToInt32 ( resp[0] );
        if ( contactCustomerIndex == i )
          Console.WriteLine ( "  " + resp[1] );
        
        for ( int k = 0; k < numberOfCustomers * 4; k++ )
        {
          // get phone k
          resp = sendRequest ( input, output, "PHONE:" + k );
          if ( ( contactCustomerIndex == i ) && Convert.ToInt32 ( resp[0] ) == j )
            Console.WriteLine ( "    " + resp[1] );
        }
      }
    }
  }
  
  public static void fasterExecution ( StreamReader input, NetworkStream output, int numberOfCustomers )
  {
    String[] resp = null;
    for ( int i = 0; i < numberOfCustomers; i++ )
    {
      // get customer i
      Console.WriteLine ( sendRequest ( input, output, "CUSTOMER:" + i )[0] );
      
      for ( int j = 0; j < numberOfCustomers * 2; j++ )
      {
        // get contact j
        resp = sendRequest ( input, output, "CONTACT:" + j );
        int contactCustomerIndex = Convert.ToInt32 ( resp[0] );
        if ( contactCustomerIndex == i )
        {
          Console.WriteLine ( "  " + resp[1] );
        
          for ( int k = 0; k < numberOfCustomers * 4; k++ )
          {
            // get phone k
            resp = sendRequest ( input, output, "PHONE:" + k );
            if ( Convert.ToInt32 ( resp[0] ) == j )
              Console.WriteLine ( "    " + resp[1] );
          }
        }
      }
    }
  }
  
  public static String[] sendRequest ( StreamReader input, NetworkStream output, String request )
  {
    counter++;
    
    // send request
    //Console.WriteLine ( "sending request \"" + request + "\"..." );
    Byte[] req = Encoding.ASCII.GetBytes ( (request + "\n").ToCharArray () );
    output.Write ( req, 0, req.Length );
    //Console.WriteLine ( "request sent" );
    
    // receive response
    //Console.WriteLine ( "waiting for response..." );
    String resp = input.ReadLine ();
    //Console.WriteLine ( "response \"" + resp + "\" received" );
    return resp.Split ( new Char[] {':'} );
  }
}